@@ -4,7 +4,8 @@ import keyword
 from jedi._compatibility import is_py3, is_py35
 from jedi import common
 from jedi.evaluate.filters import AbstractNameDefinition
-from jedi.parser.tree import Leaf
+from jedi.parser.python.tree import Leaf
+
 try:
     from pydoc_data import topics as pydoc_topics
 except ImportError:
@@ -90,9 +91,6 @@ class Keyword(object):
         self.start_pos = pos
         self.parent = evaluator.BUILTINS
 
-    def get_parent_until(self):
-        return self.parent
-
     @property
     def only_valid_as_leaf(self):
         return self.name.value in keywords_only_valid_as_leaf
